/*
**   biteme -- Echelon jammer
**
**   Copyright (C) 2001  Paul Dwerryhouse (paul@dwerryhouse.com.au)
**
**   This program is free software; you can redistribute it and/or modify
**   it under the terms of the GNU General Public License as published by
**   the Free Software Foundation; either version 2 of the License, or
**   (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**   but WITHOUT ANY WARRANTY; without even the implied warranty of
**   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**   GNU General Public License for more details.
**
**   You should have received a copy of the GNU General Public License
**   along with this program; if not, write to the Free Software
**   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdio.h>
#include <db.h>
#include <time.h>
#include <unistd.h>
#include <getopt.h>
#include <errno.h>

#define WORDFILE DATADIR "/biteme.wordlist"
#define DATABASE DATADIR "/biteme.wordlist.db"

#define MAXLEN 80

void xheader(void);
void usage(char *);
void build(void);

int main(int argc, char **argv)
{

    DB *dbp;
    DBT key, data;
    int ret, n, total;
    int c, noheader = 0;

    while (1) {
	c = getopt(argc, argv, "xb");

	if (c == -1)
	    break;
	switch (c) {
	case 'b':
	    build();		/* Doesn't return */
	    break;
	case 'x':
	    noheader = 1;
	    break;
	default:
	    usage(argv[0]);

	}
    }

    srand(time(0));

    if (!noheader)
	xheader();

    if ((ret = db_create(&dbp, NULL, 0)) != 0) {
	fprintf(stderr, "db_create: %s\n", db_strerror(ret));
	exit(1);
    }
    if ((ret =
	 dbp->open(dbp, DATABASE, NULL, DB_BTREE, DB_RDONLY, 0)) != 0) {
	dbp->err(dbp, ret, "%s", DATABASE);
	exit(1);
    }

    memset(&key, 0, sizeof(key));
    memset(&data, 0, sizeof(data));
    key.data = "total";
    key.size = sizeof("total");
    if ((ret = dbp->get(dbp, NULL, &key, &data, 0)) != 0) {
	dbp->err(dbp, ret, "DB->get");
	exit(1);
    }
    total = (*(int *) data.data);

    n = rand() % total;

    memset(&key, 0, sizeof(key));
    memset(&data, 0, sizeof(data));
    key.data = (void *) &n;
    key.size = sizeof(n);
    if ((ret = dbp->get(dbp, NULL, &key, &data, 0)) != 0) {
	dbp->err(dbp, ret, "DB->get");
	exit(1);
    }
    printf("%s", data.data);

    dbp->close(dbp, 0);
}

void xheader(void)
{
    int i, n, c;

    n = rand() % 20 + 3;

    printf("X-");
    for (i = 0; i < n; i++) {
	c = 65 + (rand() % 26) + (rand() % 2) * 32;
	printf("%c", c);
    }
    printf(": ");
}

void usage(char *s)
{
    fprintf(stderr, "Usage: %s [-x] [-b]\n", s);
    exit(1);
}

#define DATABASE DATADIR "/biteme.wordlist.db"

#define MAXLEN 80

void build(void)
{

    DB *dbp;
    DBT key, data;
    int ret, n = 0;
    FILE *fd;
    char buf[MAXLEN];

    if ((ret = db_create(&dbp, NULL, 0)) != 0) {
	fprintf(stderr, "db_create: %s\n", db_strerror(ret));
	exit(1);
    }
    if ((ret =
	 dbp->open(dbp, DATABASE, NULL, DB_BTREE, DB_CREATE, 0664)) != 0) {
	dbp->err(dbp, ret, "%s", DATABASE);
	exit(1);
    }

    if (!(fd = fopen(WORDFILE, "r"))) {
	    perror("fopen");
	    exit(1);
    }

    while (fgets(buf, MAXLEN, fd)) {
	memset(&key, 0, sizeof(key));
	memset(&data, 0, sizeof(data));
	key.data = (void *) &n;
	key.size = sizeof(n);
	data.data = buf;
	data.size = sizeof(buf);
	if ((ret = dbp->put(dbp, NULL, &key, &data, 0)) != 0) {
	    dbp->err(dbp, ret, "DB->put");
	    exit(1);
	}
	n++;
    }

    fclose(fd);

    memset(&key, 0, sizeof(key));
    memset(&data, 0, sizeof(data));
    key.data = "total";
    key.size = sizeof("total");
    data.data = (void *) &n;
    data.size = sizeof(n);
    if ((ret = dbp->put(dbp, NULL, &key, &data, 0)) != 0) {
	dbp->err(dbp, ret, "DB->put");
	exit(1);
    }

    dbp->close(dbp, 0);
    exit(0);
}
